@@ -12,29 +12,44 @@ use LWP::Protocol::https;
 sub new {
     my $class = shift;
     my %args = @_;
-    bless \%args, $class;
+
+    my $ua = LWP::UserAgent->new;
+    my %headers;
+
+    if (defined $args{api_token}) {
+        %headers = (
+            'Content-Type' => 'application/json',
+            'Authorization' => "Bearer $args{api_token}",
+        );
+    } else {
+        %headers = (
+            'Content-Type' => 'application/json',
+            'X-Auth-Key'   => $args{api_key},
+            'X-Auth-Email' => $args{email},
+        );
+    }
+
+    my $zone_id = $args{zone_id};
+    die "no zone_id provided" unless defined $zone_id;
+
+    my $base_url = "https://api.cloudflare.com/client/v4/zones/$zone_id/dns_records";
+
+    bless { ua => $ua,
+            headers => \%headers,
+            base_url => $base_url,
+          }, $class;
 }
  
 sub get_records {
     my $self = shift;
     my %args = @_;
 
-    my $email = $self->{email};
-    my $api_key = $self->{api_key};
-    my $zone_id = $self->{zone_id};
-    
-    my $uri = URI->new("https://api.cloudflare.com/client/v4/zones/$zone_id/dns_records");
+    my $uri = URI->new($self->{base_url});
     $uri->query_form(%args);
+    my $ua = $self->{ua};
 
-    my $ua = LWP::UserAgent->new;
-    my %headers = (
-        'Content-Type' => 'application/json',
-        'X-Auth-Key'   => $api_key,
-        'X-Auth-Email' => $email,
-    );
- 
     my $res = $ua->get($uri,
-        %headers,
+        %{$self->{headers}},
     );
 
     if ($res->is_success ) {
@@ -44,27 +59,16 @@ sub get_records {
     }
 }
 
-
 sub create_record {
     my $self = shift;
     my %args = @_;
 
     my $data = encode_json(\%args);
-    my $email = $self->{email};
-    my $api_key = $self->{api_key};
-    my $zone_id = $self->{zone_id};
-    
+    my $uri = URI->new($self->{base_url});
+    my $ua = $self->{ua};
 
-    my $uri = "https://api.cloudflare.com/client/v4/zones/$zone_id/dns_records";
-    my $ua = LWP::UserAgent->new;
-    my %headers = (
-        'Content-Type' => 'application/json',
-        'X-Auth-Key'   => $api_key,
-        'X-Auth-Email' => $email,
-    );
- 
     my $res = $ua->post($uri,
-        %headers,
+        %{$self->{headers}},
         Content => $data,
     );
 
@@ -81,20 +85,11 @@ sub update_record {
     my %args = @_;
 
     my $data = encode_json(\%args);
-    my $email = $self->{email};
-    my $api_key = $self->{api_key};
-    my $zone_id = $self->{zone_id};
+    my $uri = URI->new($self->{base_url} . "/$record_id");
+    my $ua = $self->{ua};
 
-    my $uri = "https://api.cloudflare.com/client/v4/zones/$zone_id/dns_records/$record_id";
-    my $ua = LWP::UserAgent->new;
-    my %headers = (
-        'Content-Type' => 'application/json',
-        'X-Auth-Key'   => $api_key,
-        'X-Auth-Email' => $email,
-    );
- 
     my $res = $ua->put($uri,
-        %headers,
+        %{$self->{headers}},
         Content => $data,
     );
 
@@ -109,20 +104,11 @@ sub delete_record {
     my $self = shift;
     my $record_id = shift;
 
-    my $email = $self->{email};
-    my $api_key = $self->{api_key};
-    my $zone_id = $self->{zone_id};
-
-    my $uri = "https://api.cloudflare.com/client/v4/zones/$zone_id/dns_records/$record_id";
-    my $ua = LWP::UserAgent->new;
-    my %headers = (
-        'Content-Type' => 'application/json',
-        'X-Auth-Key'   => $api_key,
-        'X-Auth-Email' => $email,
-    );
+    my $uri = URI->new($self->{base_url} . "/$record_id");
+    my $ua = $self->{ua};
  
     my $res = $ua->delete($uri,
-        %headers,
+        %{$self->{headers}},
     );
 
     if ($res->is_success ) {
@@ -138,11 +124,11 @@ Net::Cloudflare::DNS - DNS API for Cloudflare API v4
 
 =head1 VERSION
 
-Version 0.18
+Version 0.19
 
 =cut
 
-our $VERSION = '0.18';
+our $VERSION = '0.19';
 
 
 =head1 SYNOPSIS
@@ -158,35 +144,38 @@ My system is Ubuntu, to use the module, I have the following perl modules pre-in
 
 After then, you can run "cpanm Net::Cloudflare::DNS" to install this module.
 
-    use strict;
     use Net::Cloudflare::DNS;
     use Data::Dumper;
 
-    my $api_key = " ";
+    # zone_id for your domain
     my $zone_id = " ";
-    my $email = " ";
 
-    # new the object
+    # new the object with email and api_key
+    my $email = " ";
+    my $api_key = " ";
     my $dns = Net::Cloudflare::DNS->new(email=>$email, api_key=>$api_key, zone_id=>$zone_id);
-    my $res;
+
+    # or, new the object with bearer token
+    my $api_token = " ";
+    my $dns = Net::Cloudflare::DNS->new(api_token=>$api_token, zone_id=>$zone_id);
 
     # create record
-    $res = $dns->create_record(type=>"A", name=>"www.sample.com", content=>"74.81.81.81", proxied=>\1, ttl=>60);
+    $dns->create_record(type=>"A", name=>"www.sample.com", content=>"74.81.81.81", proxied=>\1, ttl=>60);
 
     # get records
-    $res = $dns->get_records(name=>"www.sample.com");
+    my $res = $dns->get_records(name=>"www.sample.com");
 
     # parse record id
     my $rid = $res->{result}->[0]->{id};
 
     # update record
-    $res = $dns->update_record($rid, type=>"A", name=>"www.sample.com",content=>"52.1.14.22", proxied=>\0, ttl=>60);
+    $dns->update_record($rid, type=>"A", name=>"www.sample.com",content=>"52.1.14.22", proxied=>\0, ttl=>60);
 
     # delete record
-    $res = $dns->delete_record($rid);
+    $dns->delete_record($rid);
 
-    # If it success, cloudflare's response is just resturned as a reference to you.
-    # Otherwise the method just dies, you should catch the error in the code.
+    # If it succeeds, cloudflare's response is resturned as a reference to you.
+    # Otherwise the method just dies, you could catch the error in the code.
     print Dumper $res;
 
 
@@ -195,13 +184,14 @@ After then, you can run "cpanm Net::Cloudflare::DNS" to install this module.
 =head2 new
 
     my $dns = Net::Cloudflare::DNS->new(email=>$email, api_key=>$api_key, zone_id=>$zone_id);
+    # or,
+    my $dns = Net::Cloudflare::DNS->new(api_token=>$api_token, zone_id=>$zone_id);
 
-You have to provide 3 arguments to new() method. The first is your registration email on Cloudflare. The second is your API Key, which can be 
-found on Cloudflare's management panel ("Global API Key"). The last is Zone ID, each zone has the unique ID, which can be found
-on zone's page.
+You have to provide either email+api_key or api_token along with zone_id to new() method. 
+All those values can be found on cloudflare's management panel.
 
-Please notice: You must enable zone edit permissions for this API. In management panel, when you click "Create Token", you have the
-chance to setup permissions for the zone, with which you can edit zone's DNS records. 
+Please note: You must enable zone edit permissions for this API. In management panel, when you click "Create Token", 
+you have the chance to setup permissions for the zone, with which you can edit zone's DNS records. 
  
 
 =head2 create_record
@@ -272,6 +262,9 @@ the web interface at L<https://rt.cpan.org/NoAuth/ReportBug.html?Queue=Net-Cloud
 automatically be notified of progress on your bug as I make changes.
 
 
+=head1 THANKS
+
+1. Thibault, C<< <thibault.duponchelle at gmail.com> >> for porting the bearer token authentication method.
 
 
 =head1 SUPPORT
